from nonebot.matcher import Matcher

from .. import config
from ..chatmanager import chat_manager
from ..config import config_manager


async def menu(matcher: Matcher):
    """处理聊天菜单命令"""

    # 初始化消息内容为默认菜单消息
    msg = chat_manager.menu_msg

    # 遍历自定义菜单项，添加到消息内容
    for menus in chat_manager.custom_menu:
        msg += f"\n{menus['cmd']} {menus['describe']}"

    # 根据配置添加群聊或私聊的提示信息
    msg += f"\n{'群内可以at我与我聊天，' if config_manager.config.function.enable_group_chat else '未启用群内聊天，'}{'在私聊可以直接聊天。' if config_manager.config.function.enable_private_chat else '未启用私聊聊天'}\nPowered by SuggarChat V{config.__kernel_version__}"

    # 发送最终消息内容
    await matcher.send(msg)



from dataclasses import dataclass, field
from datetime import datetime
from typing import Any

from pydantic import BaseModel, Field


class SessionTemp(BaseModel):
    message_id: int
    timestamp: datetime = Field(default_factory=datetime.now)


@dataclass
class ChatManager:
    debug: bool = False
    session_clear_group: dict[str, SessionTemp] = field(default_factory=dict)
    session_clear_user: dict[str, SessionTemp] = field(default_factory=dict)
    custom_menu: list[dict[str, str]] = field(default_factory=list)
    running_messages_poke: dict[str, Any] = field(default_factory=dict)
    menu_msg: str = "聊天功能菜单:\n" + "/聊天菜单 唤出菜单 \n"
    menu_msg += "/del_memory 丢失这个群/聊天的记忆 \n"
    menu_msg += "/enable 在群聊启用聊天 \n"
    menu_msg += "/disable 在群聊里关闭聊天\n"
    menu_msg += "/prompt <arg> [text] 设置聊群自定义补充prompt（--(show) 展示当前提示词，--(clear) 清空当前prompt，--(set) [文字]则设置提示词，e.g.:/prompt --(show)）,/prompt --(set) [text]。）\n"
    menu_msg += "/sessions指令帮助：\n"
    menu_msg += "set：覆盖当前会话为指定编号的会话\n"
    menu_msg += "del：删除指定编号的会话\n"
    menu_msg += "archive：归档当前会话\n"
    menu_msg += "clear：清空所有会话\n"
    menu_msg += "Preset帮助：\n"
    menu_msg += "/presets 列出所有读取到的模型预设\n"
    menu_msg += (
        "/set_preset 或 /设置预设 或 /设置模型预设  <预设名> 设置当前使用的预设\n"
    )
    menu_msg += "/prompts 展示当前的prompt预设\n"
    menu_msg += (
        "/choose_prompt <group/private> <预设名称> 设置群聊/私聊的全局提示词预设\n"
    )
    menu_msg += "/fakepeople <on/off>模拟聊天"
    menu_msg += "/insights [global] 获取当前群聊/私聊的用量，参数：global(可选)"


chat_manager = ChatManager()



import inspect
from collections.abc import Awaitable, Callable
from copy import deepcopy
from types import FrameType
from typing import Any, ClassVar

from nonebot import logger
from nonebot.exception import (
    FinishedException,
    NoneBotException,
    ProcessException,
    StopPropagation,
)
from pydantic import BaseModel, Field
from typing_extensions import Self

from .event import SuggarEvent
from .exception import BlockException, CancelException, PassException

"""
suggar matcher
"""


class FunctionData(BaseModel, arbitrary_types_allowed=True):
    function: Callable[..., Awaitable[Any]] = Field(...)
    signature: inspect.Signature = Field(...)
    frame: FrameType = Field(...)
    priority: int = Field(...)
    block: bool = Field(...)
    matcher: Any = Field(...)


class EventRegistry:
    _instance = None
    __event_handlers: ClassVar[dict[str, list[FunctionData]]] = {}

    def __new__(cls) -> Self:
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def register_handler(self, event_type: str, data: FunctionData):
        self.__event_handlers.setdefault(event_type, []).append(data)

    def get_handlers(self, event_type: str) -> list[FunctionData]:
        self.__event_handlers.setdefault(event_type, [])
        self.__event_handlers[event_type].sort(key=lambda x: x.priority, reverse=False)
        return self.__event_handlers[event_type]

    def _all(self) -> dict[str, list[FunctionData]]:
        return self.__event_handlers


class Matcher:
    def __init__(self, event_type: str, priority: int = 10, block: bool = True):
        """构造函数，初始化Matcher对象。
        Args:
            event_type (str): 事件类型
            priority (int, optional): 优先级。 Defaults to 10.
            block (bool, optional): 是否阻止后续事件。 Defaults to True.
        """
        self.event_type = event_type
        self.priority = priority
        self.block = block

    def handle(self):
        """
        事件处理函数注册函数
        """
        if self.priority <= 0:
            raise ValueError("事件优先级不能为0或负！")

        def wrapper(
            func: Callable[..., Awaitable[Any]],
        ):
            frame = inspect.currentframe()
            assert frame is not None, "Frame is None!!!"
            func_data = FunctionData(
                function=func,
                signature=inspect.signature(func),
                frame=frame,
                priority=self.priority,
                block=self.block,
                matcher=self,
            )
            EventRegistry().register_handler(self.event_type, func_data)
            return func

        return wrapper

    def stop_process(self):
        """
        阻止当前Suggar事件循环继续运行并立即停止当前的处理器。
        """
        raise BlockException()

    def cancel(self):
        """
        终止Nonebot层的处理器
        """
        raise FinishedException()

    def cancel_matcher(self):
        """
        停止当前Suggar事件处理并取消。
        """
        raise CancelException()

    def cancel_nonebot_process(self):
        """
        直接停止Nonebot的处理流程，不触发任何事件处理程序。
        """
        raise StopPropagation()

    def pass_event(self):
        """
        忽略当前处理器，继续处理下一个。
        """
        raise PassException()


class MatcherManager:
    @staticmethod
    async def trigger_event(*args, **kwargs):
        """
        触发特定类型的事件，并调用该类型的所有注册事件处理程序。

        参数:
        - event: SuggarEvent 对象，包含事件相关数据。
        - **kwargs: 关键字参数，传递给依赖注入系统的参数。
        - *args: 可变参数，传递给依赖注入系统的参数。
        """
        event: SuggarEvent | None = None
        for i in args:
            if isinstance(i, SuggarEvent):
                event = i
                break
        if not event:
            logger.error("事件必须被传入，但是是没有找到！")
            return
        event_type = event.get_event_type()  # 获取事件类型
        priority_tmp = 0
        logger.info(f"正在为事件: {event_type} 运行matcher!")
        # 检查是否有处理该事件类型的处理程序
        if matcher_list := EventRegistry().get_handlers(event_type):
            for matcher in matcher_list:
                if matcher.priority != priority_tmp:
                    priority_tmp = matcher.priority
                    logger.info(f"为优先级 {priority_tmp} 运行Matcher......")

                signature = matcher.signature
                frame = matcher.frame
                line_number = frame.f_lineno
                file_name = frame.f_code.co_filename
                handler = matcher.function
                session_args = [matcher.matcher, *args]
                session_kwargs = {**deepcopy(kwargs)}

                args_types = {k: v.annotation for k, v in signature.parameters.items()}
                filtered_args_types = {
                    k: v for k, v in args_types.items() if v is not inspect._empty
                }
                if args_types != filtered_args_types:
                    failed_args = list(args_types.keys() - filtered_args_types.keys())
                    logger.warning(
                        f"匹配器 {matcher.function.__name__} (File: {file_name}: Line {frame.f_lineno!s}) 有没有类型注解的参数！"
                        + f"(Args:{''.join(i + ',' for i in failed_args)}).跳过......"
                    )
                    continue
                new_args = []
                used_indices = set()
                for param_type in filtered_args_types.values():
                    for i, arg in enumerate(session_args):
                        if i in used_indices:
                            continue
                        if isinstance(arg, param_type):
                            new_args.append(arg)
                            used_indices.add(i)
                            break
                new_args_tuple = tuple(new_args)

                # 获取关键词参数类型注解
                kwparams = signature.parameters
                f_kwargs = {
                    param_name: session_kwargs[param.annotation]
                    for param_name, param in kwparams.items()
                    if param.annotation in session_kwargs
                }
                if len(new_args_tuple) != len(list(filtered_args_types)):
                    continue

                # 调用处理程序

                try:
                    logger.info(f"开始运行Matcher: '{handler.__name__}'")

                    await handler(*new_args_tuple, **f_kwargs)

                except ProcessException as e:
                    logger.info("停止Nonebot处理")
                    raise e
                except PassException:
                    logger.info(
                        f"Matcher '{handler.__name__}'(~{file_name}:{line_number}) 已跳过"
                    )
                    continue
                except CancelException:
                    logger.info("取消了Matcher处理")
                    return
                except BlockException:
                    break
                except NoneBotException:
                    raise
                except Exception as e:
                    logger.error(
                        f"运行时发生了错误 '{handler.__name__}'({file_name}:{line_number}) "
                    )
                    logger.opt(exception=e, colors=True).exception(str(e))
                    continue
                finally:
                    logger.info(f"处理器 {handler.__name__} 已结束")
                    if matcher.block:
                        break
        else:
            logger.info(f"没有为 {event_type} 事件注册的Matcher，跳过处理。")



from nonebot import MatcherGroup, on_command
from nonebot.rule import Rule

from .check_rule import (
    is_bot_admin,
    is_bot_enabled,
    is_group_admin,
    is_group_admin_if_is_in_group,
    should_respond_to_message,
)
from .handlers.add_notices import add_notices
from .handlers.chat import chat
from .handlers.choose_prompt import choose_prompt
from .handlers.debug_switchs import debug_switchs
from .handlers.del_memory import del_memory
from .handlers.disable import disable
from .handlers.enable import enable
from .handlers.fakepeople_switch import switch
from .handlers.insights import insights
from .handlers.menus import menu
from .handlers.poke_event import poke_event
from .handlers.presets import presets
from .handlers.prompt import prompt
from .handlers.recall import recall
from .handlers.sessions import sessions
from .handlers.set_preset import set_preset

base_matcher = MatcherGroup(rule=is_bot_enabled)

base_matcher.on_notice(
    priority=5,
    block=False,
).append_handler(add_notices)
base_matcher.on_notice(
    priority=5,
    block=False,
).append_handler(poke_event)
base_matcher.on_notice(
    priority=5,
    block=False,
).append_handler(recall)

base_matcher.on_message(
    block=False,
    priority=11,
    rule=Rule(should_respond_to_message, is_bot_enabled),
).append_handler(chat)

base_matcher.on_command(
    "prompt",
    priority=10,
    block=True,
    permission=is_group_admin_if_is_in_group,
).append_handler(prompt)
base_matcher.on_command(
    "presets",
    priority=10,
    block=True,
    permission=is_bot_admin,
).append_handler(presets)
base_matcher.on_command(
    "set_preset",
    aliases={"设置预设", "设置模型预设"},
    priority=10,
    block=True,
    permission=is_bot_admin,
).append_handler(set_preset)
base_matcher.on_command(
    "debug",
    priority=10,
    block=True,
    permission=is_bot_admin,
).append_handler(debug_switchs)
base_matcher.on_command(
    "autochat",
    aliases={"自动回复", "autoreply"},
    priority=10,
    block=True,
    permission=is_group_admin,
).append_handler(switch)
base_matcher.on_command(
    "choose_prompt",
    priority=10,
    block=True,
    permission=is_bot_admin,
).append_handler(choose_prompt)

base_matcher.on_command(
    "sessions",
    priority=10,
    block=True,
    permission=is_bot_admin,
).append_handler(sessions)
base_matcher.on_command(
    "del_memory",
    aliases={"失忆", "删除记忆", "删除历史消息", "删除回忆"},
    block=True,
    priority=10,
    permission=is_group_admin,
).append_handler(del_memory)
on_command(
    "enable",
    aliases={"启用聊天", "enable_chat"},
    block=True,
    priority=10,
    permission=is_group_admin,
).append_handler(enable)
on_command(
    "disable",
    aliases={"禁用聊天", "disable_chat"},
    block=True,
    priority=10,
    permission=is_group_admin,
).append_handler(disable)

base_matcher.on_command(
    "聊天菜单",
    block=True,
    aliases={"chat_menu"},
    priority=10,
).append_handler(menu)

base_matcher.on_command(
    "insights",
    aliases={"今日用量"},
    block=True,
    priority=10,
).append_handler(insights)



import asyncio
import os
import sys
import traceback
from pathlib import Path
from typing import TYPE_CHECKING

import nonebot
from dotenv import load_dotenv
from nonebot.adapters.onebot.v11 import Adapter as ONEBOT_V11Adapter
from nonebot.adapters.onebot.v11 import Bot, MessageSegment
from nonebot.log import default_format, logger_id

from amrita.config import get_amrita_config

from .admin import send_forward_msg_to_admin

if TYPE_CHECKING:
    # avoid sphinx autodoc resolve annotation failed
    # because loguru module do not have `Logger` class actually
    from loguru import Record


def default_filter(record: "Record"):
    """默认的日志过滤器，根据 `config.log_level` 配置改变日志等级。"""
    log_level = record["extra"].get("nonebot_log_level", "INFO")
    levelno = (
        nonebot.logger.level(log_level).no if isinstance(log_level, str) else log_level
    )
    return record["level"].no >= levelno


class AsyncErrorHandler:
    def write(self, message):
        self.task = asyncio.create_task(self.process(message))

    async def process(self, message):
        try:
            record = message.record
            if record["level"].name == "ERROR":
                # 处理异常 traceback
                if record["exception"]:
                    exc_info = record["exception"]
                    traceback_str = "".join(
                        traceback.format_exception(
                            exc_info.type, exc_info.value, exc_info.traceback
                        )
                    )
                else:
                    traceback_str = "无堆栈信息"

                content = (
                    f"错误信息: {record['message']}\n"
                    f"时间: {record['time']}\n"
                    f"模块: {record['name']}\n"
                    f"文件: {record['file'].path}\n"
                    f"行号: {record['line']}\n"
                    f"函数: {record['function']}\n"
                    f"堆栈信息:\n{traceback_str}"
                )

                bot = nonebot.get_bot()
                if isinstance(bot, Bot):
                    await send_forward_msg_to_admin(
                        bot,
                        "Amrita-Exception",
                        bot.self_id,
                        [MessageSegment.text(content)],
                    )

        except Exception as e:
            nonebot.logger.warning(f"发送群消息失败: {e}")


def init():
    Path("plugins").mkdir(exist_ok=True)

    load_dotenv()

    nonebot.init()

    driver = nonebot.get_driver()
    driver.register_adapter(ONEBOT_V11Adapter)

    nonebot.load_from_toml("pyproject.toml")

    load_dotenv()
    config = get_amrita_config()
    log_dir = config.log_dir
    os.makedirs(log_dir, exist_ok=True)

    # 移除 NoneBot 默认的日志处理器
    nonebot.logger.remove(logger_id)
    # 添加新的日志处理器
    nonebot.logger.add(
        sys.stdout,
        level=0,
        diagnose=True,
        format=default_format,
        filter=default_filter,
    )
    nonebot.logger.add(
        f"{log_dir}/" + "{time}.log",  # 传入函数，每天自动更新日志路径
        level=config.amrita_log_level,
        format=default_format,
        rotation="00:00",
        retention="7 days",
        encoding="utf-8",
        enqueue=True,
    )
    nonebot.logger.add(AsyncErrorHandler(), level="ERROR")



import os
from pathlib import Path

import click

from amrita.cli import (
    error,
    info,
    plugin,
    question,
    stdout_run_proc,
    success,
    warn,
)

from .cli import nb


@plugin.command()
@click.argument("name")
def install(name: str):
    """Install a plugin."""
    nb(["plugin", "install", name])


@plugin.command()
@click.argument("name")
def new(name: str):
    """Create a new plugin."""
    cwd = Path(os.getcwd())
    plugins_dir = cwd / "plugins"

    if not plugins_dir.exists():
        click.echo(error("Not in an Amrita project directory."))
        return

    plugin_dir = plugins_dir / name
    if plugin_dir.exists():
        click.echo(warn(f"Plugin {name} already exists."))
        overwrite = click.confirm(
            question("Do you want to overwrite it?"), default=False
        )
        if not overwrite:
            return

    os.makedirs(plugin_dir, exist_ok=True)

    # 创建插件文件
    with open(plugin_dir / "__init__.py", "w") as f:
        f.write(
            f"from . import {name.replace('-', '_')}\n\n__all__ = ['{name.replace('-', '_')}']\n"
        )

    with open(plugin_dir / f"{name.replace('-', '_')}.py", "w") as f:
        f.write(f"""from nonebot import on_command
from nonebot.adapters.onebot.v11 import MessageEvent

# Register your commands here
{name.replace("-", "_")} = on_command("{name}")

@{name.replace("-", "_")}.handle()
async def handle_function(event: MessageEvent):
    await {name.replace("-", "_")}.finish("Hello from {name}!")
""")

    # 创建配置文件
    with open(plugin_dir / "config.py", "w") as f:
        f.write(f"""# Configuration for {name} plugin

# Add your configuration here
""")

    click.echo(success(f"Plugin {name} created successfully!"))


@plugin.command()
@click.argument("name")
def remove(name: str):
    """Remove a plugin."""
    cwd = Path(os.getcwd())
    plugin_dir = cwd / "plugins" / name

    if not plugin_dir.exists():
        click.echo(error(f"Plugin {name} does not exist."))
        return

    confirm = click.confirm(
        question(f"Are you sure you want to remove plugin '{name}'?"), default=False
    )
    if not confirm:
        return

    # 删除插件目录
    import shutil

    shutil.rmtree(plugin_dir)
    click.echo(success(f"Plugin {name} removed successfully!"))


@plugin.command()
def list_plugins():
    """List all plugins."""
    cwd = Path(os.getcwd())
    plugins_dir = cwd / "plugins"
    plugins = []
    stdout = stdout_run_proc(["uv", "run", "pip", "freeze"])
    freeze_str = [
        "(Package) " + (i.split("=="))[0]
        for i in (stdout).split("\n")
        if i.startswith("nonebot-plugin") or i.startswith("amrita-plugin")
    ]
    plugins.extend(freeze_str)

    if not plugins_dir.exists():
        click.echo(error("Not in an Amrita project directory."))
        return

    if not plugins_dir.is_dir():
        click.echo(error("Plugins directory is not a directory."))
        return

    plugins.extend(
        [
            "(Local) " + item.name.replace(".py", "")
            for item in plugins_dir.iterdir()
            if (
                not (item.name.startswith("-") or item.name.startswith("_"))
                and (item.is_dir() or item.name.endswith(".py"))
            )
        ]
    )

    if not plugins:
        click.echo(info("No plugins found."))
        return

    click.echo(success("Available plugins:"))
    for pl in plugins:
        click.echo(f"  - {pl}")



from nonebot import require
from sqlalchemy import String, Text
from sqlalchemy.orm import MappedColumn, mapped_column

require("nonebot_plugin_orm")
from nonebot_plugin_orm import Model


class GroupBlacklist(Model):
    group_id: MappedColumn[str] = mapped_column(String(50), primary_key=True)
    reason: MappedColumn[str] = mapped_column(Text)

    __tablename__ = "group_blacklist"


class PrivateBlacklist(Model):
    user_id: MappedColumn[str] = mapped_column(String(50), primary_key=True)
    reason: MappedColumn[str] = mapped_column(Text)

    __tablename__ = "private_blacklist"



import json
import time
from datetime import datetime, timedelta
from typing import Any, Literal, overload

from nonebot_plugin_orm import AsyncSession, Model, get_session
from pydantic import BaseModel as B_Model
from pydantic import Field
from sqlalchemy import (
    JSON,
    BigInteger,
    Boolean,
    DateTime,
    ForeignKey,
    Index,
    Integer,
    String,
    Text,
    UniqueConstraint,
    delete,
    insert,
    select,
    text,
    update,
)
from sqlalchemy.orm import Mapped, mapped_column
from typing_extensions import Self

from ..config import config_manager
from .lock import database_lock

# Pydantic 模型


class BaseModel(B_Model):
    def __str__(self) -> str:
        return json.dumps(self.model_dump(), ensure_ascii=True)

    def __repr__(self) -> str:
        return self.__str__()

    def __getitem__(self, key: str) -> Any:
        return self.model_dump()[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.__setattr__(key, value)


class ImageUrl(BaseModel):
    url: str = Field(..., description="图片URL")


class ImageContent(BaseModel):
    type: Literal["image_url"] = "image_url"
    image_url: ImageUrl = Field(..., description="图片URL")


class TextContent(BaseModel):
    type: Literal["text"] = "text"
    text: str = Field(..., description="文本内容")


class Message(BaseModel):
    role: Literal["user", "assistant", "system"] = Field(..., description="角色")
    content: str | list[TextContent | ImageContent] = Field(..., description="内容")


class ToolResult(BaseModel):
    role: Literal["tool"] = Field(default="tool", description="角色")
    name: str = Field(..., description="工具名称")
    content: str = Field(..., description="工具返回内容")
    tool_call_id: str = Field(..., description="工具调用ID")


class MemoryModel(BaseModel):
    messages: list[Message | ToolResult] = Field(default_factory=list)
    time: float = Field(default_factory=time.time, description="时间戳")


class InsightsModel(BaseModel):
    date: str = Field(
        default_factory=lambda: datetime.now().strftime("%Y-%m-%d"), description="日期"
    )
    token_input: int = Field(..., description="输入token使用量")
    token_output: int = Field(..., description="输出token使用量")
    usage_count: int = Field(..., description="聊天请求次数")

    @classmethod
    async def get(cls) -> Self:
        date_now = datetime.now().strftime("%Y-%m-%d")
        async with database_lock(date_now):
            async with get_session() as session:
                await cls._delete_expired(
                    days=config_manager.config.usage_limit.global_insights_expire_days,
                    session=session,
                )
                if (
                    insights := (
                        await session.execute(
                            select(GlobalInsights).where(
                                GlobalInsights.date == date_now
                            )
                        )
                    ).scalar_one_or_none()
                ) is None:
                    stmt = insert(GlobalInsights).values(date=date_now)
                    await session.execute(stmt)
                    insights = (
                        await session.execute(
                            select(GlobalInsights).where(
                                GlobalInsights.date == date_now
                            )
                        )
                    ).scalar_one()
                session.add(insights)
                instance = cls.model_validate(insights, from_attributes=True)
            return instance

    async def save(self):
        """保存数据"""
        async with database_lock(self.date):
            async with get_session() as session:
                await self._delete_expired(
                    days=config_manager.config.usage_limit.global_insights_expire_days,
                    session=session,
                )
                stmt = select(GlobalInsights).where(GlobalInsights.date == self.date)
                if ((await session.execute(stmt)).scalar_one_or_none()) is None:
                    stmt = insert(GlobalInsights).values(
                        **{
                            k: v
                            for k, v in self.model_dump().items()
                            if hasattr(GlobalInsights, k)
                        }
                    )
                    await session.execute(stmt)
                    await session.commit()
                else:
                    stmt = (
                        update(GlobalInsights)
                        .where(GlobalInsights.date == self.date)
                        .values(
                            **{
                                k: v
                                for k, v in self.model_dump().items()
                                if hasattr(GlobalInsights, k)
                            }
                        )
                    )
                    await session.execute(stmt)
                    await session.commit()

    @staticmethod
    async def _delete_expired(*, days: int, session: AsyncSession) -> int:
        """
        删除过期的记录

        Args:
            days: 保留天数，超过此天数的记录将被删除
        """
        # 计算截止日期
        cutoff_date = datetime.now() - timedelta(days=days)

        # 删除过期记录
        stmt = delete(GlobalInsights).where(
            GlobalInsights.date < cutoff_date.strftime("%Y-%m-%d")
        )
        result = await session.execute(stmt)
        await session.commit()
        return result.rowcount


# Sqlalchemy 模型


class GlobalInsights(Model):
    __tablename__ = "suggarchat_global_insights"
    date: Mapped[str] = mapped_column(
        String(64),
        primary_key=True,
        default=lambda: datetime.now().strftime("%Y-%m-%d"),
    )
    token_input: Mapped[int] = mapped_column(
        BigInteger, default=0, server_default=text("0")
    )
    token_output: Mapped[int] = mapped_column(
        BigInteger, default=0, server_default=text("0")
    )
    usage_count: Mapped[int] = mapped_column(Integer, default=0)


class Memory(Model):
    __tablename__ = "suggarchat_memory_data"
    id: Mapped[int] = mapped_column(Integer, primary_key=True, autoincrement=True)
    ins_id: Mapped[int] = mapped_column(BigInteger, nullable=False)
    is_group: Mapped[bool] = mapped_column(Boolean, nullable=False, default=False)
    memory_json: Mapped[dict[str, Any]] = mapped_column(
        JSON,
        default=MemoryModel().model_dump(),
        nullable=False,
        server_default=text("'{}'"),
    )
    sessions_json: Mapped[list[dict[str, Any]]] = mapped_column(
        JSON,
        default=[],
        nullable=False,
        server_default=text("'[]'"),
    )
    time: Mapped[datetime] = mapped_column(
        DateTime, default=datetime.now, nullable=False
    )
    usage_count: Mapped[int] = mapped_column(Integer, default=0)
    input_token_usage: Mapped[int] = mapped_column(BigInteger, default=0)
    output_token_usage: Mapped[int] = mapped_column(BigInteger, default=0)
    __table_args__ = (
        UniqueConstraint("ins_id", "is_group", name="uq_ins_id_is_group"),
        Index("idx_ins_id", "ins_id"),
        Index("idx_is_group", "is_group"),
    )


class GroupConfig(Model):
    __tablename__ = "suggarchat_group_config"
    id: Mapped[int] = mapped_column(
        Integer,
        primary_key=True,
        autoincrement=True,
    )
    group_id: Mapped[int] = mapped_column(
        BigInteger,
        ForeignKey("suggarchat_memory_data.ins_id"),
        nullable=False,
    )
    enable: Mapped[bool] = mapped_column(Boolean, default=True)
    prompt: Mapped[str] = mapped_column(Text, default="")
    fake_people: Mapped[bool] = mapped_column(Boolean, default=False)
    last_updated: Mapped[datetime] = mapped_column(DateTime, default=datetime.now)

    __table_args__ = (
        UniqueConstraint("group_id", name="uq_suggarchat_config_group_id"),
        Index("idx_suggarchat_group_id", "group_id"),
    )


@overload
async def get_or_create_data(
    *, session: AsyncSession, ins_id: int, for_update: bool = False
) -> Memory: ...
@overload
async def get_or_create_data(
    *,
    session: AsyncSession,
    ins_id: int,
    is_group: bool = True,
    for_update: bool = False,
) -> tuple[GroupConfig, Memory]: ...


async def get_or_create_data(
    *,
    session: AsyncSession,
    ins_id: int,
    is_group: bool = False,
    for_update: bool = False,
) -> Memory | tuple[GroupConfig, Memory]:
    async with database_lock(ins_id, is_group):
        stmt = select(Memory).where(
            Memory.ins_id == ins_id, Memory.is_group == is_group
        )
        stmt = stmt.with_for_update() if for_update else stmt
        result = await session.execute(stmt)
        if not (memory := result.scalar_one_or_none()):
            stmt = insert(Memory).values(ins_id=ins_id, is_group=is_group)
            await session.execute(stmt)
            await session.commit()
            stmt = select(Memory).where(
                Memory.ins_id == ins_id, Memory.is_group == is_group
            )
            stmt = stmt.with_for_update() if for_update else stmt
            memory = (await session.execute(stmt)).scalar_one()
        session.add(memory)
        if not is_group:
            return memory
        stmt = select(GroupConfig).where(GroupConfig.group_id == ins_id)
        stmt = stmt.with_for_update() if for_update else stmt
        result = await session.execute(stmt)
        if not (group_config := result.scalar_one_or_none()):
            stmt = insert(GroupConfig).values(group_id=ins_id)
            await session.execute(stmt)
            await session.commit()
            stmt = select(GroupConfig).where(GroupConfig.group_id == ins_id)
            group_config = (await session.execute(stmt)).scalar_one()
        session.add(group_config)
        return group_config, memory



from importlib import metadata
from typing import Literal

from nonebot import get_plugin_config
from pydantic import BaseModel


class AmritaConfig(BaseModel):
    log_dir: str = "logs"
    admin_group: int
    disabled_builtin_plugins: list[Literal["chat", "manager", "perm", "menu"]] = []
    amrita_log_level: Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"] = (
        "WARNING"
    )
    public_group: int = 0  # Bot对外展示公开群
    bot_name: str = "Amrita"
    rate_limit: int = 5  # Bot请求速率限制(间隔秒)


def get_amrita_config() -> AmritaConfig:
    return get_plugin_config(AmritaConfig)


def get_amrita_version() -> str:
    version = "unknown"
    try:
        version = metadata.version("amrita")
    except metadata.PackageNotFoundError:
        pass
    return version



from .models import PluginData


def generate_menu(plugins: list[PluginData]) -> list[str]:
    head = "# 菜单\n\n" + "> 这是菜单列表，包含所有可用的功能和用法。\n\n"
    head += "## 模块列表\n\n"
    for plugin in plugins:
        if not plugin.metadata or not plugin.matcher_grouping:
            continue
        plugin_name = plugin.metadata.name
        plugin_desc = plugin.metadata.description or "无描述"
        head += f"\n\n- **{plugin_name}**: {plugin_desc}"

    menu_datas: list[str] = [head.strip()]
    for plugin in plugins:
        if not plugin.matcher_grouping or not plugin.metadata:
            continue

        plugin_title = f"## {plugin.metadata.name}\n"
        plugin_description = (
            f"> {plugin.metadata.description}\n\n"
            if plugin.metadata.description
            else ""
        )
        plugin_markdown = plugin_title + plugin_description
        for matchers in plugin.matcher_grouping.values():
            for matcher_data in matchers:
                plugin_markdown += (
                    f"- **{matcher_data.name}**: {matcher_data.description}"
                )
                if matcher_data.usage:
                    plugin_markdown += f"\n    - 用法: `{matcher_data.usage}`"
                plugin_markdown += "\n\n"
        menu_datas.append(plugin_markdown.strip())

    return menu_datas



from nonebot import logger

from .chatmanager import chat_manager
from .config import Config, ConfigManager, config_manager
from .on_event import on_before_chat, on_before_poke, on_chat, on_event, on_poke
from .utils.admin import send_to_admin
from .utils.libchat import (
    AdapterManager,
    ModelAdapter,
    get_chat,
    tools_caller,
)
from .utils.llm_tools.manager import ToolsManager
from .utils.llm_tools.models import (
    FunctionDefinitionSchema,
    FunctionParametersSchema,
    FunctionPropertySchema,
    ToolContext,
    ToolData,
    ToolFunctionSchema,
)
from .utils.memory import get_memory_data
from .utils.models import InsightsModel
from .utils.protocol import ToolChoice
from .utils.tokenizer import Tokenizer, hybrid_token_count


class Menu:
    """
    Menu 类用于通过注册菜单项来构建菜单。
    """

    def reg_menu(self, cmd_name: str, describe: str, args: str = "") -> "Menu":
        """
        注册一个新的菜单项。

        参数:
        - cmd_name (str): 菜单项的命令名称。
        - describe (str): 菜单项的描述。

        返回:
        - Menu: 返回 Menu 类的实例，支持方法链式调用。
        """
        chat_manager.menu_msg += f"/{cmd_name} {args} 说明：{describe} \n"
        return self

    @property
    def menu(self) -> str:
        """
        获取菜单项。
        """
        return chat_manager.menu_msg


class Admin:
    """
    管理员管理类，负责处理与管理员相关的操作，如发送消息、错误处理和管理员权限管理。
    """

    config: Config

    def __init__(self):
        """
        构造函数
        """
        self.config = config_manager.ins_config

    async def send_with(self, msg: str) -> "Admin":
        """
        异步发送消息给管理员。

        参数:
        - msg (str): 要发送的消息内容。

        返回:
        - Admin: 返回Admin实例，支持链式调用。
        """
        await send_to_admin(msg)
        return self

    async def send_error(self, msg: str) -> "Admin":
        """
        异步发送错误消息给管理员，并记录错误日志。

        参数:
        - msg (str): 要发送的错误消息内容。

        返回:
        - Admin: 返回Admin实例，支持链式调用。
        """
        logger.error(msg)
        await send_to_admin(msg)
        return self

    def is_admin(self, user_id: str) -> bool:
        """
        检查用户是否是管理员。

        参数:
        - user_id (str): 用户ID。

        返回:
        - bool: 用户是否是管理员。
        """
        return int(user_id) in self.config.admin.admins

    def add_admin(self, user_id: int) -> "Admin":
        """
        添加新的管理员用户ID到配置中。

        参数:
        - user_id (int): 要添加的用户ID。

        返回:
        - Admin: 返回Admin实例，支持链式调用。
        """
        self.config.admin.admins.append(user_id)
        return self._save_config_to_toml()

    def set_admin_group(self, group_id: int) -> "Admin":
        """在Amrita不适用。"""
        return self

    def _save_config_to_toml(self):
        self.config.save_to_toml(config_manager.toml_config)
        self.config = config_manager.ins_config
        return self


class Chat:
    """
    Chat 类用于处理与LLM相关操作，如获取消息。
    """

    config: Config

    def __init__(self):
        """
        构造函数
        """
        self.config = config_manager.ins_config

    async def get_msg(self, prompt: str, message: list):
        """
        获取LLM响应

        :param prompt[str]: 提示词
        :param message[list]: 消息列表
        """

        message.insert(0, {"role": "assistant", "content": prompt})
        return await self.get_msg_on_list(message)

    async def get_msg_on_list(self, message: list):
        """
        获取LLM响应

        :param message[list]: 消息列表
        """

        return await get_chat(messages=message)

    async def call_tools(
        self,
        messages: list,
        tools: list,
        tool_choice: ToolChoice | None = None,
    ):
        """
        调用工具

        :param messages: 消息列表
        :param tools: 工具列表
        :param tool_choice: 工具选择参数
        """
        return await tools_caller(
            messages=messages, tools=tools, tool_choice=tool_choice
        )


__all__ = [
    "AdapterManager",
    "Admin",
    "Chat",
    "ConfigManager",
    "FunctionDefinitionSchema",
    "FunctionParametersSchema",
    "FunctionPropertySchema",
    "InsightsModel",
    "Menu",
    "ModelAdapter",
    "Tokenizer",
    "ToolContext",
    "ToolData",
    "ToolFunctionSchema",
    "ToolsManager",
    "config_manager",
    "get_memory_data",
    "hybrid_token_count",
    "on_before_chat",
    "on_before_poke",
    "on_chat",
    "on_event",
    "on_poke",
    "tools_caller",
]



import os
from pathlib import Path

import nonebot
import toml

from amrita.config import get_amrita_config


def load_plugins():
    nonebot.logger.info("Loading built-in plugins...")
    config = get_amrita_config()
    for name in (Path(__file__).parent.parent / "plugins").iterdir():
        if name in config.disabled_builtin_plugins:
            continue
        nonebot.logger.info(f"Loading plugin {name.name}...")
        nonebot.require(f"amrita.plugins.{name.name}")
    nonebot.logger.info("Loading plugins......")
    from amrita.cmds.cli import PyprojectFile

    meta = PyprojectFile.model_validate(toml.load("pyproject.toml"))
    for plugin in meta.tool.nonebot.plugins:
        nonebot.logger.info(f"Loading plugin {plugin}...")
        try:
            nonebot.require(plugin)
        except Exception as e:
            nonebot.logger.error(f"Failed to load plugin {plugin}: {e}")
    nonebot.logger.info("Loading local pluings...")
    try:
        nonebot.load_plugins(str(Path(os.getcwd()) / "plugins"))
    except Exception as e:
        nonebot.logger.warning(f"Failed to load local plugins: {e}")



DOTENV = """ENVIRONMENT=dev
DRIVER=~fastapi
PORT=8080
HOST=127.0.0.1
LOCALSTORE_USE_CWD=true
DATABASE_URL=aiosqlite:///db.sqlite3
LOG_DIR=logs
BOT_NAME=Amrita
RATE_LIMIT=5"""
DOTENV_DEV = """LOG_LEVEL=DEBUG"""
DOTENV_PROD = """"""
GITIGNORE = """# Created by https://www.toptal.com/developers/gitignore/api/python
# Edit at https://www.toptal.com/developers/gitignore?templates=python

### Python ###
# Byte-compiled / optimized / DLL files
__pycache__/
*.py[cod]
*$py.class

# C extensions
*.so

# Distribution / packaging
.Python
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
wheels/
share/python-wheels/
*.egg-info/
.installed.cfg
*.egg
MANIFEST

# PyInstaller
#  Usually these files are written by a python script from a template
#  before PyInstaller builds the exe, so as to inject date/other infos into it.
*.manifest
*.spec

# Installer logs
pip-log.txt
pip-delete-this-directory.txt

# Unit test / coverage reports
htmlcov/
.tox/
.nox/
.coverage
.coverage.*
.cache
nosetests.xml
coverage.xml
*.cover
*.py,cover
.hypothesis/
.pytest_cache/
cover/

# Translations
*.mo
*.pot

# Django stuff:
*.log
local_settings.py
db.sqlite3
db.sqlite3-journal

# Flask stuff:
instance/
.webassets-cache

# Scrapy stuff:
.scrapy

# Sphinx documentation
docs/_build/

# PyBuilder
.pybuilder/
target/

# Jupyter Notebook
.ipynb_checkpoints

# IPython
profile_default/
ipython_config.py

# pyenv
#   For a library or package, you might want to ignore these files since the code is
#   intended to run in multiple environments; otherwise, check them in:
# .python-version

# pipenv
#   According to pypa/pipenv#598, it is recommended to include Pipfile.lock in version control.
#   However, in case of collaboration, if having platform-specific dependencies or dependencies
#   having no cross-platform support, pipenv may install dependencies that don't work, or not
#   install all needed dependencies.
#Pipfile.lock

# poetry
#   Similar to Pipfile.lock, it is generally recommended to include poetry.lock in version control.
#   This is especially recommended for binary packages to ensure reproducibility, and is more
#   commonly ignored for libraries.
#   https://python-poetry.org/docs/basic-usage/#commit-your-poetrylock-file-to-version-control
#poetry.lock

# pdm
#   Similar to Pipfile.lock, it is generally recommended to include pdm.lock in version control.
#pdm.lock
#   pdm stores project-wide configurations in .pdm.toml, but it is recommended to not include it
#   in version control.
#   https://pdm.fming.dev/#use-with-ide
.pdm.toml

# PEP 582; used by e.g. github.com/David-OConnor/pyflow and github.com/pdm-project/pdm
__pypackages__/

# Celery stuff
celerybeat-schedule
celerybeat.pid

# SageMath parsed files
*.sage.py

# Environments
.env
.venv
env/
venv/
ENV/
env.bak/
venv.bak/

# Spyder project settings
.spyderproject
.spyproject

# Rope project settings
.ropeproject

# mkdocs documentation
/site

# mypy
.mypy_cache/
.dmypy.json
dmypy.json

# Pyre type checker
.pyre/

# pytype static type analyzer
.pytype/

# Cython debug symbols
cython_debug/

# PyCharm
#  JetBrains specific template is maintained in a separate JetBrains.gitignore that can
#  be found at https://github.com/github/gitignore/blob/main/Global/JetBrains.gitignore
#  and can be added to the global gitignore or merged into this file.  For a more nuclear
#  option (not recommended) you can uncomment the following to ignore the entire idea folder.
#.idea/

### Python Patch ###
# Poetry local configuration file - https://python-poetry.org/docs/configuration/#local-configuration
poetry.toml

# ruff
.ruff_cache/

# LSP config files
pyrightconfig.json

# End of https://www.toptal.com/developers/gitignore/api/python"""

README = """
# {project_name}

## How to start

1. generate project using `amrita init` .
2. create your plugin using `nb plugin create` .
3. writing your plugins under `{project_name}/plugins` folder.
4. run your bot using `amrita run --reload` .

## Documentation

[AmritaDocs](https://amrita.suggar.top)

See Nonebot [Docs](https://nonebot.dev/)
"""
